
# q-expansion of newform 46.8.a.c, downloaded from the LMFDB on 05 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 46
weight = 8
poly_data = [46, -121, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[-81, 5, 1], 5], [[-81, -5, 1], 5]]

hecke_ring_character_values = None
aps_data = [[8, 0, 0], [-5, 2, -1], [-182, -9, 15], [-456, -63, -49], [-3786, -153, 281], [-179, 603, 78], [-9416, 1593, -1071], [-15314, 1080, -1896], [-12167, 0, 0], [1859, -4743, 12030], [7759, -2286, -12733], [40646, -5391, 25977], [-147499, 801, 10792], [1772, -24390, -27148], [73987, -30726, 30695], [453046, -72036, -1694], [220912, 113742, 33758], [1354130, 5166, -21640], [-2354878, 58617, 54711], [-1666989, -175176, -132111], [-2903541, -111285, 16536], [-1321330, -89802, -202736], [-2694272, 32535, 347731], [-653904, 276156, -366378], [-165452, 223137, 747365], [917886, -382626, 336554], [-5392108, 444339, -678811], [4379362, 397674, -31790], [-3270784, 417672, 87400], [8585688, -343179, -135623], [8474461, -29376, 951687], [-157653, 1001070, 821055], [-20695888, 1496826, -1544468], [-6488075, 633402, -2318495], [-9089390, -1615698, 481180], [10114291, -2073528, -1518327], [-21519616, -782658, 4403974], [20055989, 1226358, -134431], [-77958984, -74106, -546518], [8236234, -4481496, -1947280], [-31537059, 1529676, -2607613], [50692726, 4241781, 4572257], [-40217862, 965637, 2814137], [126919075, -30591, 2107270], [-39760693, -3094209, 6524248], [-21695114, 4161753, -422447], [50726916, -10083006, 4226854], [11187156, -5003928, -1484332], [-132746056, 5436045, -6115079], [-52285764, 9682812, -6804832], [-1054211, -862803, 10975016], [-86777181, -13315554, 8302827], [-61302850, -2903481, -62059], [-211377800, -7645221, -7962521], [-181828335, 7604577, -15177380], [-8571230, 10822734, -21178064], [-410222629, 10654425, 96182], [72100232, 2641824, 13118112], [395691775, 3064167, -12153108], [-273234464, 7816527, 1375511], [-290314014, -7725105, -6063707], [137679360, -3347514, 6346390], [282440456, 12532518, 1700994], [371868143, -6409548, 9815625], [335398892, 3238344, -8847286], [167377294, -37408050, -12156410], [669742145, 2656890, -30411291], [-439071720, 32104908, -28095314], [398604768, 23751468, 42918548], [-747557517, -4494357, 20908940], [-404989105, -34107993, 25667988], [-35038182, -44596314, 14208228], [-776100276, -4192425, 26106285], [-374772198, 50266521, 15343661], [-525688996, -30043872, 25808402], [-289813156, 41838624, -58299716], [-797453600, -25772202, 7610786], [-230015091, 19132101, -106097812], [-746919550, 32701095, 64821913], [-54122269, 53070219, 1205708], [-324360804, -127281132, -2119866], [-548560476, 118749771, -6997007], [11951520, 114546798, -45324882], [-397563550, 18881631, -110140135], [505541149, 57399246, 78583849], [1967829347, -42523092, 36863709], [-529926018, -46543716, -89318236], [2218106532, -81260163, 33148601], [499672995, -87295437, 111315468], [471199596, -118823544, 111400708], [-1170490938, -76437468, -66389660], [-1895633340, 64264347, 72498541], [1393056169, 69459138, -126242439], [587682581, 13850802, 146130181], [1556588695, 7224336, -160562787], [2946105278, -22886370, -301328], [1570447719, -83792025, -120100428], [-1246380104, 156944160, 36634438], [-1373072328, -31794993, 293528723], [-4199257993, -38172933, -152638808], [3189467521, -299556630, 78648293], [933073876, 138680622, 72435318], [1394231760, -226702926, 66998800], [-2283615190, 172889487, -286856247], [-2456019716, -145196955, -195476735], [6112829715, 67262697, -125928606], [-1530260997, 85417668, -55759735], [6167221898, 150375276, -218502128], [-186803784, 90234432, -327080264], [4235063427, 185072517, -253803458], [1038858876, -127130418, 62486846], [1072421020, -491024457, 380653009], [3225456450, -226363230, 256145094], [50216876, 159831648, -399815362], [-4520548572, -231554394, -371433370], [8937535368, 68914476, 136684706], [-4193686754, 135925605, 120277251], [3390697911, 85304070, 277220831], [-6028865519, 331990533, -13322752], [-1614431384, -58730913, -175659565], [-5794881630, -313194330, 75733344], [1115939565, -290507049, -428656982], [5173127710, 310640922, 233375708], [-11114982515, -223174800, 252803339], [2214722492, -270564084, 514176108], [4491665772, 574602345, -67236873], [-738158048, 242966151, -270972959], [-4430861808, 419419980, -492459660], [5712285474, -542614905, -288330129], [1702556988, -456034977, -161647379], [-1013675807, -81097452, 856807287], [4008814596, -278858079, 135675615], [1454166064, 38892690, -652257274], [2745813346, -376350867, -418381387], [7659730527, 220221279, -916043940], [850427678, 728895879, -74890507], [-7307328964, -571348602, 770381982], [-7883966048, -615851568, 1012845770], [2300693434, -273007800, 313695358], [-6506226874, -368963568, -894459380], [-12888407077, -807566202, 112628079], [-4206704794, -425129760, -547595264], [90553675, 456398856, -648553135], [-2024887908, -706912668, 166776174], [6325763638, 840702474, 21975026], [-10105346308, -212393529, 531543193], [-18300052454, -247366170, 42674478], [-5104405659, -62657739, 1334575944], [8961440163, -432822096, 981231241], [20384848849, 616059774, 943415781], [-8114902190, 888518898, 392168270], [-7580355992, -542113749, -95456293], [6458623468, 1028473488, 608803592], [-10630488609, 410145264, 1201179381], [-9703759038, 409009806, -786434126], [6637203710, -726721920, 622139790], [9140177548, -116679195, 361802359], [-6233463187, -1287570753, 1974093394], [-18381789910, 2040466941, -597403729], [11692923488, 1473032763, -787192067], [13474937551, 903839904, -2709647467], [12132550270, -443654118, -1282177314], [379065625, -416155608, -324004589], [8330854938, -2686187574, 878122078], [-18772092622, -183805317, 598963837], [29652807330, -139062132, 96572934], [-38032994792, -630660168, 1658144808], [3813248522, 879154776, 584786144]]
